##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_bgp_api.tcl
#
# Purpose:
#     A script development library containing BGP APIs for test automation with the Ixia chassis.
#
# Author:
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_bgp_config
#    - emulation_bgp_route_config
#    - emulation_bgp_control
#    - emulation_bgp_info
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and
#     parsedashedargds.tcl
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_bgp_config { args } {

    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_bgp_config $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable bgp_neighbor_handles_array
    variable bgp_route_handles_array

    ::ixia::utrackerLog $procName $args

    # Arguments
    set man_args {
        -mode        CHOICES delete disable enable modify reset
    }

    set opt_args {
        -md5_enable                   CHOICES 0 1
        -md5_key                      ANY
        -port_handle                  REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -handle
        -ip_version                   CHOICES 4 6
                                      DEFAULT 4
        -local_ip_addr                IPV4
        -gateway_ip_addr              IP
        -remote_ip_addr               IPV4
        -local_ipv6_addr              IPV6
        -remote_ipv6_addr             IPV6
        -local_addr_step              IP
        -remote_addr_step             IP
        -next_hop_enable              CHOICES 0 1
                                      FLAG
        -next_hop_ip                  IP
        -enable_4_byte_as             CHOICES 0 1
                                      DEFAULT 0
        -local_as                     RANGE   0-4294967295
        -local_as_mode                CHOICES fixed increment
                                      DEFAULT fixed
        -remote_as                    RANGE   0-4294967295
        -local_as_step                RANGE   0-4294967295
                                      DEFAULT 1
        -update_interval              RANGE   0-65535
        -count                        NUMERIC
                                      DEFAULT 1
        -local_router_id              IPV4
        -local_router_id_step         IPV4
                                      DEFAULT 0.0.0.1
        -vlan                         CHOICES 0 1
        -vlan_id                      RANGE   0-4095
        -vlan_id_mode                 CHOICES fixed increment
                                      DEFAULT increment
        -vlan_id_step                 RANGE   0-4096
                                      DEFAULT 1
        -vlan_user_priority           RANGE   0-7
        -vpi                          RANGE   0-255
        -vci                          RANGE   0-65535
        -vpi_step                     RANGE   0-255
        -vci_step                     RANGE   0-65535
        -atm_encapsulation            CHOICES VccMuxIPV4Routed
                                      CHOICES VccMuxIPV6Routed
                                      CHOICES VccMuxBridgedEthernetFCS
                                      CHOICES VccMuxBridgedEthernetNoFCS
                                      CHOICES LLCRoutedCLIP
                                      CHOICES LLCBridgedEthernetFCS
                                      CHOICES LLCBridgedEthernetNoFCS
        -interface_handle             
        -retry_time                   NUMERIC
        -hold_time                    NUMERIC
        -neighbor_type                CHOICES internal external
        -graceful_restart_enable      FLAG
        -restart_time                 RANGE   0-10000000
        -stale_time                   RANGE   0-10000000
        -tcp_window_size              RANGE   0-10000000
        -staggered_start_enable       FLAG
        -staggered_start_time         RANGE   0-10000000
        -retries                      RANGE   0-10000000
        -local_router_id_enable
        -active_connect_enable        FLAG
        -netmask                      RANGE   1-128
        -mac_address_start            MAC
        -mac_address_step             MAC
                                      DEFAULT 0000.0000.0001
        -ipv4_mdt_nlri                FLAG
        -ipv4_unicast_nlri            FLAG
        -ipv4_multicast_nlri          FLAG
        -ipv4_mpls_nlri               FLAG
        -ipv4_mpls_vpn_nlri           FLAG
        -ipv6_unicast_nlri            FLAG
        -ipv6_multicast_nlri          FLAG
        -ipv6_mpls_nlri               FLAG
        -ipv6_mpls_vpn_nlri           FLAG
        -local_loopback_ip_addr       IP
        -local_loopback_ip_prefix_length NUMERIC
        -local_loopback_ip_addr_step  IP
        -remote_loopback_ip_addr      IP
        -remote_loopback_ip_addr_step IP
        -ttl_value                    NUMERIC
        -updates_per_iteration        RANGE   0-10000000
        -bfd_registration             CHOICES 0 1
                                      DEFAULT 0
        -bfd_registration_mode        CHOICES single_hop multi_hop
                                      DEFAULT multi_hop
        -override_existence_check     CHOICES 0 1
                                      DEFAULT 0
        -override_tracking            CHOICES 0 1
                                      DEFAULT 0
        -no_write                     FLAG
        -vpls                         CHOICES 0 1 disabled vpn no_vpn
                                      DEFAULT 0      
        -vpls_nlri                    FLAG
    }
    
    # If vpls_nlri (or other params of type FLAG) is specified, move it at the end of the $args list
    # Otherwise you might see a parse dashed args error like this <can't use non-numeric string as operand of "!">
    set args [::ixia::move_flags_last $args [list -next_hop_enable -vpls_nlri -ipv4_unicast_nlri\
            -ipv4_mpls_vpn_nlri -ipv6_mpls_nlri -graceful_restart_enable -staggered_start_enable\
            -active_connect_enable -ipv4_mdt_nlri -ipv4_multicast_nlri -ipv4_mpls_nlri\
            -ipv6_unicast_nlri -ipv6_multicast_nlri -ipv6_mpls_vpn_nlri]]
    
    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_bgp_config $args $man_args $opt_args]
        if {![catch {set log [keylget returnList log]}]} {
            keylset returnList log "ERROR in $procName: $log"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_bgp_route_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_bgp_route_config $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable bgp_neighbor_handles_array
    variable bgp_route_handles_array

    ::ixia::utrackerLog $procName $args
    # Arguments
    set man_args {
        -handle REGEXP  ^.+$
        -mode   CHOICES add remove
    }
    
    set opt_args {
        -end_of_rib                          CHOICES 0 1
        -route_handle                        REGEXP  ^.+$
        -l3_site_handle                      REGEXP  ^.+$
        -ip_version                          CHOICES 4 6
                                             DEFAULT 4
        -prefix                              IP
                                             CHOICES all
        -netmask                             IP
        -ipv6_prefix_length                  RANGE   1-128
        -num_sites                           NUMERIC
        -num_routes                          NUMERIC
        -num_labels                          RANGE   1-65535
                                             DEFAULT 1
        -num_labels_type                     CHOICES list single_value
                                             DEFAULT single_value
        -max_route_ranges                    NUMERIC
                                             DEFAULT 1
        -mtu                                 RANGE   0-65535
                                             DEFAULT 1500
        -prefix_from                         RANGE   0-128
        -prefix_to                           RANGE   0-128
        -prefix_step                         NUMERIC
        -prefix_step_across_vrfs             IP
        -packing_from                        RANGE   0-65535
        -packing_to                          RANGE   0-65535
        -route_ip_addr_step                  IP
        -origin_route_enable                 FLAG
        -originator_id_enable                VCMD ::ixia::validate_flag_choice_0_1
        -origin                              CHOICES igp egp incomplete
        -as_path                             REGEXP  ^.+$
        -as_path_set_mode                    CHOICES include_as_seq include_as_seq_conf include_as_set include_as_set_conf no_include prepend_as
                                             DEFAULT include_as_seq
        -next_hop                            IP
        -next_hop_enable                     CHOICES 0 1
                                             FLAG
                                             DEFAULT 1
        -next_hop_set_mode                   CHOICES same manual
                                             DEFAULT same
        -next_hop_ip_version                 CHOICES 4 6
        -next_hop_mode                       CHOICES fixed increment incrementPerPrefix
        -multi_exit_disc                     NUMERIC
        -local_pref                          NUMERIC
                                             DEFAULT 0
        -atomic_aggregate                    FLAG
        -aggregator                          REGEXP  ^[0-9]{1,5}:[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}$
        -communities_enable                  FLAG
        -communities                         VCMD ::ixia::validate_bgp_communities
        -ext_communities                     REGEXP  ^.+$
        -enable_local_pref                   CHOICES 0 1
        -enable_as_path                      CHOICES 0 1
        -originator_id                       IP
        -cluster_list_enable                 CHOICES 0 1
                                             FLAG
        -cluster_list                        REGEXP ^([0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}.[0-9]{1,3}\s*)+|([0-9]+\s*)+$
        -label_block_offset                  RANGE 0-65535
                                             DEFAULT 0
        -label_block_offset_type             CHOICES list single_value
                                             DEFAULT single_value
        -label_value                         NUMERIC
                                             DEFAULT 16
        -label_id                            NUMERIC
                                             DEFAULT 0
        -label_value_end                     NUMERIC
        -label_value_type                    CHOICES list single_value
                                             DEFAULT single_value
        -label_incr_mode                     CHOICES fixed rd prefix
        -label_step                          NUMERIC
                                             DEFAULT 1
        -l2_start_mac_addr                   MAC
        -l2_mac_incr                         CHOICES 0 1
                                             DEFAULT 0
        -l2_mac_count                        NUMERIC
                                             DEFAULT 1
        -l2_enable_vlan                      CHOICES 0 1
                                             DEFAULT 0
        -l2_vlan_id                          RANGE 0-65535
                                             DEFAULT 1
        -l2_vlan_id_incr                     NUMERIC
        -l2_vlan_incr                        CHOICES 0 1 2 3 no_increment parallel_increment inner_first outer_first
                                             DEFAULT 0
        -rd_type                             RANGE 0-2
                                             DEFAULT 0
        -rd_admin_value                      IP
                                             NUMERIC
        -rd_admin_value_step                 IP
                                             NUMERIC
        -rd_admin_value_step_across_vrfs     IP
                                             NUMERIC
        -rd_assign_value                     NUMERIC
        -rd_assign_value_step                NUMERIC
        -rd_assign_value_step_across_vrfs    NUMERIC
        -rd_admin_step                       IP
                                             NUMERIC
        -rd_assign_step                      NUMERIC
        -rd_count                            NUMERIC
                                             DEFAULT 1
        -rd_count_per_vrf                    NUMERIC
                                             DEFAULT 1
        -site_id                             RANGE 0-65535
                                             DEFAULT 0
        -site_id_step                        RANGE 0-65535
                                             DEFAULT 0
        -target_type                         CHOICES as ip
        -target                              IP
                                             NUMERIC
        -target_count                        NUMERIC
        -target_assign                       NUMERIC
        -target_step                         IP
                                             NUMERIC
        -target_inner_step                   IP
                                             NUMERIC
        -target_assign_step                  NUMERIC
        -target_assign_inner_step            NUMERIC
        -import_target_type                  CHOICES as ip
        -import_target                       IP
                                             NUMERIC
        -import_target_assign                NUMERIC
        -import_target_step                  IP
                                             NUMERIC
        -import_target_inner_step            IP
                                             NUMERIC
        -import_target_assign_step           NUMERIC
        -import_target_assign_inner_step     NUMERIC
        -ipv4_unicast_nlri                   FLAG
        -ipv4_multicast_nlri                 FLAG
        -ipv4_mpls_nlri                      FLAG
        -ipv4_mpls_vpn_nlri                  FLAG
        -ipv6_unicast_nlri                   FLAG
        -ipv6_multicast_nlri                 FLAG
        -ipv6_mpls_nlri                      FLAG
        -ipv6_mpls_vpn_nlri                  FLAG
        -default_mdt_ip                      IP
        -default_mdt_ip_incr                 IP
        -enable_generate_unique_routes       FLAG
        -enable_partial_route_flap           FLAG
        -enable_route_flap                   FLAG
        -enable_traditional_nlri             CHOICES 0 1
                                             DEFAULT 1
        -flap_down_time                      NUMERIC
        -flap_up_time                        NUMERIC
        -partial_route_flap_from_route_index NUMERIC
        -partial_route_flap_to_route_index   NUMERIC
        -no_write                            FLAG
        -vpls                                FLAG
        -vpls_nlri                           FLAG
    }
    
    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_bgp_route_config $args $man_args $opt_args]
        if {![catch {set log [keylget returnList log]}]} {
            keylset returnList log "ERROR in $procName: $log"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_bgp_control { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_bgp_control $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable bgp_neighbor_handles_array

    ::ixia::utrackerLog $procName $args

    # Arguments
    set man_args {
        -mode        CHOICES link_flap restart start stop statistic
    }

    set opt_args {
        -handle
        -link_flap_up_time    RANGE  0-10000000
        -link_flap_down_time  RANGE  0-10000000
        -port_handle          REGEXP ^[0-9]+/[0-9]+/[0-9]+$
    }

     if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_bgp_control $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName: [keylget returnList log]"
        }
        keylset returnList clicks [format "%u" $retValueClicks]
        keylset returnList seconds [format "%u" $retValueSeconds]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_bgp_info { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_bgp_info $args\}]

        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    variable bgp_neighbor_handles_array
    variable bgp_route_handles_array

    ::ixia::utrackerLog $procName $args

    # Arguments
    set man_args {
        -mode    CHOICES stats clear_stats settings neighbors labels
        -handle
    }

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_bgp_info $args $man_args]
        if {![catch {set log [keylget returnList log]}]} {
            keylset returnList log "ERROR in $procName: $log"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}
